@@ -23,11 +23,8 @@ class SpGlue : public SpBase<typename T1::elem_type, SpGlue<T1, T2, spglue_type>
   typedef typename T1::elem_type                   elem_type;
   typedef typename get_pod_type<elem_type>::result pod_type;
   
-  //static const bool is_row = (is_same_type<spglue_type, spglue_times>::value) ? T1::is_row : false;
-  //static const bool is_col = (is_same_type<spglue_type, spglue_times>::value) ? T2::is_col : false;
-  
-  static const bool is_row = false;
-  static const bool is_col = false;
+  static const bool is_row = ( (T1::is_row || T2::is_row) && is_spglue_elem<spglue_type>::value ) || ( (is_spglue_times<spglue_type>::value || is_spglue_times2<spglue_type>::value) ? T1::is_row : false );
+  static const bool is_col = ( (T1::is_col || T2::is_col) && is_spglue_elem<spglue_type>::value ) || ( (is_spglue_times<spglue_type>::value || is_spglue_times2<spglue_type>::value) ? T2::is_col : false );
   
   arma_inline  SpGlue(const T1& in_A, const T2& in_B);
   arma_inline  SpGlue(const T1& in_A, const T2& in_B, const elem_type in_aux);
